import struct
import logging
import time

from .access import dump


class ModbusTcp:
    def __init__(self, host, port, slave_id):
        self.host = host
        self.port = port
        if not slave_id:
            slave_id = 255
        self.slave_id = slave_id

        import umodbus.client.tcp as modbus
        self.protocol = modbus

    def connect(self):
        import socket
        self.connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.connection.settimeout(5)
        self.connection.connect((self.host, self.port))

    def send(self, request):
        self.connection.send(request)

    def receive(self, request):
        header = self.receive_n(6)
        seq, _, count = struct.unpack('>3H', header)
        sent_seq = struct.unpack('>H', request[:2])[0]
        if seq != sent_seq:
            logging.warn('Sequence mismatch: sent %s, received %s', sent_seq, seq)
        response = header + self.receive_n(count)

        logging.debug('← < %s > %s bytes', dump(response), len(response))

        return self.protocol.parse_response_adu(response, request)

    def receive_n(self, n):
        data = bytes()
        while len(data) < n:
            len_before = len(data)
            data += self.connection.recv(n - len(data))
            if len(data) == len_before:
                time.sleep(0.1)
        return data

    def close(self):
        self.connection.close()

    def perform_accesses(self, accesses, definitions):
        for access in accesses:
            if access.write:
                access.write_registers_send(self)
            else:
                access.read_registers_send(self)

        for access in accesses:
            if access.write:
                access.write_registers_receive(self)
            else:
                access.read_registers_receive(self)
                access.print_values(definitions)

        return self



import unittest
import logging

from modbus_cli.definitions import Definitions

logging.basicConfig(level=logging.DEBUG)


class TestDefinitions(unittest.TestCase):

    def test_parse(self):
        it = Definitions(False)
        it.parse(['tests/simple.modbus'])
        self.assertEqual({'a_register': 'i@100/4H:a_presenter'}, it.registers)
        self.assertEqual({':a_presenter': {0: 'x', 1: 'y'}}, it.presenters)

    def test_parse_silent(self):
        it = Definitions(True)
        it.parse(['tests/simple.modbus'])
        self.assertEqual({'a_register': 'i@100/4H:a_presenter'}, it.registers)
        self.assertEqual({':a_presenter': {0: 'x', 1: 'y'}}, it.presenters)


if __name__ == '__main__':
    unittest.main()



import logging

from .access import dump


class ModbusRtu:
    def __init__(self, device, baud, parity, stop_bits, slave_id, timeout):
        from serial import PARITY_EVEN, PARITY_ODD, PARITY_NONE
        parity_opts = {'e': PARITY_EVEN, 'o': PARITY_ODD, 'n': PARITY_NONE}
        self.device = device
        self.timeout = timeout
        self.baud = baud
        self.parity = parity_opts[parity]
        self.stop_bits = stop_bits
        if slave_id is None:
            slave_id = 1
        self.slave_id = slave_id

        import umodbus.client.serial.rtu as modbus
        self.protocol = modbus

    def connect(self):
        from serial import Serial

        logging.debug("Serial port %s. Parameters: %s baud, %s stop bit(s), parity: %s, timeout %ss.",
                      self.device,
                      self.baud,
                      self.stop_bits,
                      self.parity,
                      self.timeout,
                      )

        self.connection = Serial(port=self.device, baudrate=self.baud, parity=self.parity,
                                 stopbits=self.stop_bits, bytesize=8, timeout=self.timeout)

    def send(self, request):
        self.connection.write(request)

    def receive(self, request):
        response = self.connection.read(2)
        if len(response) != 2:
            raise RuntimeError('timeout')
        slave_id, function = response

        try:
            if function in (1, 2, 3, 4):
                # Functions with variable size
                response += self.connection.read(1)
                count = 2 + response[-1]
                response += self.connection.read(count)
            elif function in (5, 6, 15, 16):
                # Function with fixed size
                response += self.connection.read(6)
            elif function & 0x80:
                response += self.connection.read(3)
            else:
                response += self.connection.read(1024)
                raise NotImplementedError('RTU function {}'.format(function))
        finally:
            logging.debug('← < %s > %s bytes', dump(response), len(response))

        return self.protocol.parse_response_adu(response, request)

    def close(self):
        self.connection.close()

    def perform_accesses(self, accesses, definitions):
        for access in accesses:
            access.perform(self)
            if not access.write:
                access.print_values(definitions)

        return self



import unittest
from unittest.mock import Mock
import logging

from modbus_cli.access import parse_accesses, Access

logging.basicConfig(level=logging.DEBUG)


def mocked_modbus():
    modbus = Mock()
    modbus.slave_id = 42

    modbus.protocol = Mock()
    modbus.protocol.read_input_registers = Mock(return_value=[])
    modbus.protocol.read_holding_registers = Mock(return_value=[])
    modbus.protocol.read_discrete_inputs = Mock(return_value=[])
    modbus.protocol.read_coils = Mock(return_value=[])

    modbus.protocol.write_multiple_registers = Mock(return_value=[])
    modbus.protocol.write_multiple_coils = Mock(return_value=[])

    return modbus


class TestAccess(unittest.TestCase):

    def test_defaults(self):
        it = parse_accesses(['123'], None)
        self.assertEqual(1, len(it))

        it = it[0]
        self.assertEqual('h', it.modbus_type)
        self.assertEqual(123, it.address())
        self.assertEqual('!H', it.pack_type())
        self.assertEqual(None, it.presenter())
        self.assertEqual(1, it.size())

    def test_full(self):
        it = parse_accesses(['i@123/<4H:STATUS'], None)
        self.assertEqual(1, len(it))

        it = it[0]
        self.assertEqual('i', it.modbus_type)
        self.assertEqual(123, it.address())
        self.assertEqual('<4H', it.pack_type())
        self.assertEqual(':STATUS', it.presenter())
        self.assertEqual(4, it.size())

    def test_grouping(self):
        it = parse_accesses(['123', '124'], None)
        self.assertEqual(1, len(it))

        it = it[0]
        self.assertEqual('h', it.modbus_type)
        self.assertEqual(123, it.address())
        self.assertEqual('!H', it.pack_type())
        self.assertEqual(None, it.presenter())
        self.assertEqual(2, it.size())

    def test_read_input_registers(self):
        modbus = mocked_modbus()
        modbus.receive = Mock(return_value=[0x1234, 0x5678])
        access = Access('i', [123, 124], ['!H', '!H'])
        access.perform(modbus)

        modbus.protocol.read_input_registers.assert_called_once_with(42, 123, 2)
        modbus.send.assert_called_once()

        self.assertEqual([(0x1234, ), (0x5678, )], access.values)

    def test_read_holding_registers(self):
        modbus = mocked_modbus()
        modbus.receive = Mock(return_value=[0x1234, 0x5678])
        access = Access('h', [123, 124], ['!H', '!H'])
        access.perform(modbus)

        modbus.protocol.read_holding_registers.assert_called_once_with(42, 123, 2)
        modbus.send.assert_called_once()

        self.assertEqual([(0x1234, ), (0x5678, )], access.values)

    def test_read_discrete_inputs(self):
        modbus = mocked_modbus()
        modbus.receive = Mock(return_value=[1, 0])
        access = Access('d', [123, 124], ['!B', '!B'])
        access.perform(modbus)

        modbus.protocol.read_discrete_inputs.assert_called_once_with(42, 123, 2)
        modbus.send.assert_called_once()

        self.assertEqual([(1,), (0,)], access.values)

    def test_read_coils(self):
        modbus = mocked_modbus()
        modbus.receive = Mock(return_value=[1, 0])
        access = Access('c', [123, 124], ['!B', '!B'])
        access.perform(modbus)

        modbus.protocol.read_coils.assert_called_once_with(42, 123, 2)
        modbus.send.assert_called_once()

        self.assertEqual([(1,), (0,)], access.values)

    def test_write_registers(self):
        modbus = mocked_modbus()
        access = Access('h', [123, 124], ['!H', '!H'], values=['10', '11'])
        access.perform(modbus)

        modbus.protocol.write_multiple_registers.assert_called_once_with(42, 123, [10, 11])
        modbus.send.assert_called_once()

    def test_write_coils(self):
        modbus = mocked_modbus()
        access = Access('c', [123, 124], ['!B', '!B'], values=['1', '0'])
        access.perform(modbus)

        modbus.protocol.write_multiple_coils.assert_called_once_with(42, 123, [1, 0])
        modbus.send.assert_called_once()

    def test_presenter(self):
        pass

    def test_endianness_read(self):
        modbus = mocked_modbus()

        # given these two 16 bit registers, what do we interpret?
        modbus.receive = Mock(return_value=[0x1122, 0x3344])

        def perform(byte_order, fields):
            addresses = list(range(len(fields)))
            access = Access('h', addresses, fields, byte_order=byte_order)
            access.perform(modbus)
            return access.values

        # big endian 16/32 bit fields
        self.assertEqual(perform('be', ['>H', '>H']), [(0x1122, ), (0x3344, )])
        self.assertEqual(perform('be', ['>I']), [(0x11223344, )])

        # little endian 16/32 bit fields
        self.assertEqual(perform('le', ['<H', '<H']), [(0x2211, ), (0x4433, )])
        self.assertEqual(perform('le', ['<I']), [(0x44332211, )])

        # mixed endian 16/32 bit fields
        self.assertEqual(perform('mixed', ['<H', '<H']), [(0x1122, ), (0x3344, )])
        self.assertEqual(perform('mixed', ['<I']), [(0x33441122, )])

    def test_endianness_write(self):
        modbus = mocked_modbus()

        # given these user inputs, what 16 bit registers are actually committed?
        values16 = ["0x1122", "0x3344"]
        values32 = ["0x11223344"]

        def perform(byte_order, fields, values):
            addresses = list(range(len(fields)))
            access = Access('h', addresses, fields, values=values, byte_order=byte_order)
            access.perform(modbus)
            return modbus.protocol.write_multiple_registers.call_args.args[2]

        # big endian 16/32 bit fields
        self.assertEqual(perform('be', ['>H', '>H'], values16), [0x1122, 0x3344])
        self.assertEqual(perform('be', ['>I'], values32), [0x1122, 0x3344])

        # little endian 16/32 bit fields
        self.assertEqual(perform('le', ['<H', '<H'], values16), [0x2211, 0x4433])
        self.assertEqual(perform('le', ['<I'], values32), [0x4433, 0x2211])

        # mixed endian 16/32 bit fields
        self.assertEqual(perform('mixed', ['<H', '<H'], values16), [0x1122, 0x3344])
        self.assertEqual(perform('mixed', ['<I'], values32), [0x3344, 0x1122])


if __name__ == '__main__':
    unittest.main()



from setuptools import setup


def readme():
    with open('README.rst') as f:
        return f.read()


setup(name='modbus_cli',
      version='0.1.9',
      description='Command line tool to access Modbus devices',
      long_description=readme(),
      long_description_content_type='text/x-rst',
      url='http://github.com/favalex/modbus-cli',
      author='Gabriele Favalessa',
      author_email='favalex@gmail.com',
      license='MPL 2.0',
      packages=['modbus_cli'],
      scripts=['bin/modbus'],
      install_requires=['umodbus', 'colorama'],
      zip_safe=False,
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: End Users/Desktop',
          'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3',
          'Topic :: Utilities',
      ])



import re
import logging

REGISTER_RE = re.compile(r'([cCdhHi]@)?(\d+)(/[^:|]*)?([:|].*)?')


class Definitions:
    def __init__(self, silent):
        self.registers = {}
        self.presenters = {}
        self.silent = silent

    def parse(self, filenames):
        for filename in filenames:
            if filename:
                with open(filename) as f:
                    self.filename = filename
                    self.line = 0
                    accumulated_line = ''
                    for line in f:
                        self.line += 1
                        if line[0].isspace():
                            accumulated_line += line
                        else:
                            self.parse_line(accumulated_line)
                            accumulated_line = line
                    self.parse_line(accumulated_line)
        if not self.silent: 
            logging.info('Parsed %d registers definitions from %d files', len(self.registers), len(filenames))

    def parse_line(self, line):
        if not line:
            return

        line = line.split('#')[0]
        line = line.strip()
        if not line:
            return

        if line[0] in ':|':
            name, values = self.parse_presenter(line)
            self.presenters[name] = values
        else:
            parts = line.split()
            if len(parts) == 2:
                name, definition = parts

                if REGISTER_RE.match(definition):
                    self.registers[name] = definition
                else:
                    logging.warning('%s:%d:Invalid definition %r for register %r. Skipping it.',
                                    self.filename, self.line, definition, name)
            else:
                logging.warning('%s:%d:Invalid definition %r. Skipping it.',
                                self.filename, self.line, line)

    def parse_presenter(self, line):
        parts = line.split()

        name = parts[0]

        values = {}
        for definition in parts[1:]:
            value, symbol = definition.split('=')

            values[int(value, 0)] = symbol

        return name, values
